@@ -1,4 +1,4 @@
-from onvif.client import ONVIFService, ONVIFCamera, SUPPORTED_SERVICES
+from onvif.client import ONVIFService, ONVIFCamera, SERVICES
 from onvif.exceptions import ONVIFError, ERR_ONVIF_UNKNOWN, \
         ERR_ONVIF_PROTOCOL, ERR_ONVIF_WSDL, ERR_ONVIF_BUILD
 from onvif import cli
@@ -6,5 +6,5 @@ from onvif import cli
 __all__ = ( 'ONVIFService', 'ONVIFCamera', 'ONVIFError',
             'ERR_ONVIF_UNKNOWN', 'ERR_ONVIF_PROTOCOL',
             'ERR_ONVIF_WSDL', 'ERR_ONVIF_BUILD',
-            'SUPPORTED_SERVICES', 'cli'
+            'SERVICES', 'cli'
            )
