@@ -9,7 +9,10 @@ from argparse import ArgumentParser, ArgumentError, REMAINDER
 
 from suds import MethodNotFound
 from suds.sax.text import Text
-from onvif import ONVIFCamera, ONVIFService, ONVIFError, SUPPORTED_SERVICES
+from onvif import ONVIFCamera, ONVIFService, ONVIFError
+from definition import SERVICES
+
+SUPPORTED_SERVICES = SERVICES.keys()
 
 class ThrowingArgumentParser(ArgumentParser):
     def error(self, message):
@@ -18,6 +21,7 @@ class ThrowingArgumentParser(ArgumentParser):
 
 def success(message):
     print 'True: ' + str(message)
+
 def error(message):
     print 'False: ' + str(message)
 
@@ -63,7 +67,7 @@ class ONVIFCLI(Cmd):
             args.params = '{}'
 
         # params must be a dictionary format string
-        match = re.match(r"^.*(\{.*\}).*$", args.params)
+        match = re.match(r"^.*?(\{.*\}).*$", args.params)
         if not match:
             return error('Invalid params')
 
@@ -82,7 +86,7 @@ class ONVIFCLI(Cmd):
         except Exception as err:
             return error(err)
 
-        if isinstance(response, Text):
+        if isinstance(response, (Text, bool)):
             return success(response)
         # Try to convert instance to dictionary
         try:
